
#include "nanojit.h"

namespace nanojit
{


void CodeBlockList::addBlockToList(CodeBlock* blockToAdd) {
	NanoAssert(blockToAdd->next() == NULL);
	blockToAdd->_next = _head;
	_head = blockToAdd;
}

CodeBlock* CodeBlockList::removeFirstBlockFromList() {
	NanoAssert(!isEmpty());
	NanoAssert(!_head->terminator()->isExec());
	CodeBlock* newCodeBlock = _head;
	_head = newCodeBlock->next();
	newCodeBlock->_next = NULL;
	return newCodeBlock;
}

bool CodeBlockList::isMember(CodeBlock* block) {
	CodeBlock* currentBlock = _head;
	while (currentBlock != NULL) {
		if (currentBlock == block)
			return true;

		currentBlock = currentBlock->_next;
	}

	return false;
}

CodeBlock* CodeBlockList::findParent(CodeBlock* block) {
	if (block == _head)
		return NULL;

	CodeBlock* currentBlock = _head;
	while (currentBlock != NULL) {
		CodeBlock* nextBlockInList = currentBlock->_next;
		if (nextBlockInList == block)
			return currentBlock;

		currentBlock = nextBlockInList;
	}

	NanoAssert(0); // block not in list
	return NULL;
}

void CodeBlockList::removeNextBlockFromList(CodeBlock* block) {
	CodeBlock* pNext = block->_next;
	block->_next = block->_next->_next;
	pNext->_next = NULL;
}

} // namespace nanojit
