

#include "nanojit.h"


namespace nanojit
{

LirStorage::LirStorage()
: _allocator(NULL)
, _head(NULL)
, _current(NULL)
, _storageElementIndex(0)
, _contentList() {
}

LirStorage::~LirStorage() {
}

void LirStorage::initialize(Allocator* allocator) {
	_allocator              = allocator;
	_head                   = NULL;
	_current                = NULL;
	_storageElementIndex    = 0;

	_contentList.clear();
}

void LirStorage::shutdown() {
	_allocator              = NULL;
	_head                   = NULL;
	_current                = NULL;
	_storageElementIndex    = 0;

	_contentList.clear();
}

void LirStorage::clear() {
	_head                   = NULL;
	_current                = _head;
	_storageElementIndex    = SEGMENT_STORAGE_SIZE;

	_contentList.clear();
}

void LirStorage::createNewSegment() {
	StorageSegment* pNewSegment = _allocator->alloc<StorageSegment>();
	pNewSegment->next = NULL;

	if (_current != NULL)
		_current->next = pNewSegment;

	_current = pNewSegment;

	_storageElementIndex = 0;
}

NANOJIT_FORCEINLINE LInsElement* LirStorage::getNewElement() {
	if (_storageElementIndex == SEGMENT_STORAGE_SIZE)
	{
		createNewSegment();
	}

	LInsElement* pNewElement = &_current->storage[_storageElementIndex];
	_storageElementIndex++;

	return pNewElement;
}

LInsElement* LirStorage::insertNewElement() {
	LInsElement* element  = getNewElement();
	_contentList.insertTail(element->node());
	return element;
}

LInsElement* LirStorage::peekNextElement() {
	if (_storageElementIndex == SEGMENT_STORAGE_SIZE) {
		createNewSegment();
	}

	return &_current->storage[_storageElementIndex];
}

void LirStorage::moveBefore(LInsElement* pElementToMove, LInsElement* pElementTarget) {
	Iterator iteratorToMove(_contentList.getIteratorAt(pElementToMove));
	_contentList.remove(iteratorToMove);
	Iterator iteratorTarget(_contentList.getIteratorAt(pElementTarget));
	_contentList.insertBefore(iteratorTarget, pElementToMove->node());
}

void LirStorage::moveAfter(LInsElement* pElementToMove, LInsElement* pElementTarget) {
	Iterator iteratorToMove(_contentList.getIteratorAt(pElementToMove));
	_contentList.remove(iteratorToMove);
	Iterator iteratorTarget(_contentList.getIteratorAt(pElementTarget));
	_contentList.insertAfter(iteratorTarget, pElementToMove->node());
}


}
